
//----------------------------------------------------------------------
// Copyright 1997, Gary Wu, gwu@cco.caltech.edu.
// ALL RIGHTS RESERVED
//
// File Name:   SoShadedLineSet.h
// Description: SoShadedLineSet, an Inventor node that uses texture
//              mapping hardware to simulate rendering of illuminated
//              field lines.
//----------------------------------------------------------------------

// ---------------------------------------------------------------------
// $Id: SoShadedLineSet.h,v 1.4 1998/02/27 22:13:20 gwu Exp $
// $Source: /usr/multires/projects/SoShadedLineSet/SoShadedLineSet.h,v $
// ---------------------------------------------------------------------
// $Log: SoShadedLineSet.h,v $
// Revision 1.4  1998/02/27 22:13:20  gwu
// Recovered the "lost" sensor to detect changes in the lineset data.
//
// Revision 1.3  1998/02/06 02:35:15  gwu
// Updated the comments.  Added the man page in SoShadedLineSet.txt.
//
// Revision 1.2  1998/02/06 01:30:35  gwu
// Updated comment fields.  Put in version information placeholders.
//
// Initial version: gwu - CS174c Project, Spring 1997
// Revisions: 
//          2-4-98: gwu - Added proper destructor
//                        Build display lists for state change
//                        Used glBindTextureEXT to speed up texture swaps
// ---------------------------------------------------------------------




#ifndef __SoShadedLineSet_h__
#define __SoShadedLineSet_h__


#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/fields/SoSFBool.h>
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoMFInt32.h>
#include <Inventor/nodes/SoNonIndexedShape.h>
#include <Inventor/elements/SoCoordinateElement.h>


#include "gTexture.h"

#include <vector.h>


class SoSensor;

class SoShadedLineSet : public SoNonIndexedShape
{

  SO_NODE_HEADER( SoShadedLineSet );

private:

  gTexture m_Texture;

  GLuint m_TextureMode_List;
  GLuint m_TextureBinding;

  int m_UpdateCache;
  float** m_TexCoords;
  int m_NumTexCoords;

  enum { LIST_X = 0, LIST_Y = 1, LIST_Z = 2 };
  int* m_SegmentList[3];

  enum { NUM_ALPHA_SEGMENTS = 20 };
  int* m_VertexAlphaIndex;
  int m_AlphaIndex;

  static const SoCoordinateElement* s_CoordElement;
  static int SortByX( const void* x0, const void* x1 );
  static int SortByY( const void* x0, const void* x1 );
  static int SortByZ( const void* x0, const void* x1 );



  static void fieldSensorCB( void* data, SoSensor* );

  void UpdateCache( const SoCoordinateElement* el );


public:
  
  // Fields
  SoMFInt32 numVertices;
  SoSFVec3f L;
  SoSFVec3f V;
  SoSFBool  stream;
  
  // Initialize the class
  static void initClass( void );

  // Constructor
  SoShadedLineSet( void );


protected:
  virtual void GLRender( SoGLRenderAction* action );

  virtual void generatePrimitives( SoAction* action );

  virtual void computeBBox( SoAction* action, SbBox3f& box, SbVec3f& center );


private:
  // Destructor
  virtual ~SoShadedLineSet();


};




#endif // __SoShadedLineSet_h__
