
/*
 * Copyright 1997, Gary Wu, gwu@cco.caltech.edu.
 * ALL RIGHTS RESERVED
 */

/*
  Initial version: gwu - CS174c Project, Spring 1997
  Revisions: 
           2-4-98: gwu - Added proper destructor
	                 Build display lists for state change
			 Used glBindTextureEXT to speed up texture swaps
			 */

// $Id: gTexture.h,v 1.2 1998/02/06 01:30:35 gwu Exp $
// $Source: /usr/multires/projects/SoShadedLineSet/gTexture.h,v $
// $Log: gTexture.h,v $
// Revision 1.2  1998/02/06 01:30:35  gwu
// Updated comment fields.  Put in version information placeholders.
//


#ifndef __gTexture_h__
#define __gTexture_h__


#include <assert.h>
#include <math.h>

#include <iostream.h>

#include <GL/gl.h>


const double PI = 3.1415926535897;


class gTexture
{
private:
  const int TEXTURE_SIZE = 128;  // including border
  enum { R = 0, G = 1, B = 2, A = 3 };

  GLubyte m_Texture[TEXTURE_SIZE][TEXTURE_SIZE][4];

  gTexture();


public:
  gTexture( float ka, float kd, float kr, float shininess )
  {

    assert( 0. <= ka && ka <= 1. );
    assert( 0. <= kd && kd <= 1. );
    assert( 0. <= kr && kr <= 1. );
    float k = ka + kd + kr;
    assert( 0. <= k  &&  k <= 1. );

    int i = 0;
    int j = 0;
    for( i = 0; i < TEXTURE_SIZE; ++i )
    {
      for( j = 0; j < TEXTURE_SIZE; ++j )
      {

	float x = ( (float) j + 1. ) / (TEXTURE_SIZE + 1.);
	float y = ( (float) i + 1. ) / (TEXTURE_SIZE + 1.);

	float LT = 2. * x - 1.;
	float VT = 2. * y - 1.;

	float intensity = 0.;
	intensity += ka;
	intensity += kd * sqrt( 1. - LT*LT );
	intensity += kr * pow( fabs( LT*VT - sqrt( 1. - LT*LT ) * sqrt( 1. - VT*VT ) ), 
			       shininess * 255. );

	if( intensity < 0. || 1. < intensity )
	{
	  cout << "intensity: " << intensity << '\n';
	  cout.flush();

	  if( intensity < 0. )
	    intensity = 0.;
	  if( 1. < intensity )
	    intensity = 1.;
	}
	assert( 0. <= intensity && intensity <= 1. );

	int c = intensity * 255;
	assert( 0 <= c && c <= 255 );

	m_Texture[i][j][R] = (GLubyte) c;
	m_Texture[i][j][G] = (GLubyte) c;
	m_Texture[i][j][B] = (GLubyte) c;
	m_Texture[i][j][A] = 127;

      }
    }
  }


  const GLubyte* Texture( void ) const
  { return &m_Texture[0][0][0]; }

  int TextureSize( void ) const
  { return TEXTURE_SIZE; }

    
};



#endif // __gTexture_h__
