//--------------------------------------------------------------
// Author: Gary Wu, gwu@cco.caltech.edu
//
// Program:     VectorField
// File Name:   main.C
// Description: Demonstrates the use of SoShadedLineSet
//--------------------------------------------------------------

static char rcsid[] = "$Id: main.C,v 1.3 1998/02/06 02:35:15 gwu Exp $";

// ---------------------------------------------------------------------
// $Id: main.C,v 1.3 1998/02/06 02:35:15 gwu Exp $
// $Source: /usr/multires/projects/SoShadedLineSet/main.C,v $
// ---------------------------------------------------------------------
// $Log: main.C,v $
// Revision 1.3  1998/02/06 02:35:15  gwu
// Updated the comments.  Added the man page in SoShadedLineSet.txt.
//
// Revision 1.2  1998/02/06 01:30:36  gwu
// Updated comment fields.  Put in version information placeholders.
//
// ---------------------------------------------------------------------


#include <assert.h>
#include <iostream.h>


#include <Inventor/SoInput.h>

#include <Inventor/nodes/SoSeparator.h>

#include <Inventor/Xt/SoXt.h> 
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h> 


#include "SoShadedLineSet.h"




SoSeparator* ReadFile( const char* filename );





//-----------------------------------
// int main( int argc, char* argv[] ) 
//   The main executable function.
//-----------------------------------
int main( int argc, char* argv[] ) 
{

  // Initialize Inventor. This returns a main window to use.    
  // If unsuccessful, exit.    

  // pass the app name    
  Widget myWindow = SoXt::init(argv[0]); 
  if( myWindow == NULL ) 
    return 1;


  // Initialize the non-built-in Inventor objects
  SoShadedLineSet::initClass();



  // Build the viewer in the application's main window
  SoXtExaminerViewer* myViewer = new SoXtExaminerViewer(myWindow);


  // Initialize the global variables, and register
  // the appropriate callback functions.
  // The input file name (if any) is passed along.
  if( argc <= 1 )
  {
  }
  else
  {
    SoSeparator* inputScene = ReadFile( argv[1] );
    if( !inputScene )
      return 1;
    assert( inputScene );
    myViewer->setSceneGraph( inputScene );
  }

  myViewer->setBackgroundColor( SbColor( 0., 0., 0. ) );

  SoXt::show( myWindow );  // Display main window
  myViewer->show();        // Display the Examiner Viewer

  SoXt::mainLoop();      // Main Inventor event loop

  return 0;
}









//--------------------------------------------------
// ReadFile
//   Creates a scene graph from file input.
//   This code was taken from The Inventor Mentor,
//     with modifications for using C++ I/O streams.
//--------------------------------------------------
SoSeparator* ReadFile( const char* filename )
{
  // Open the input file
  SoInput mySceneInput;
  if( !mySceneInput.openFile( filename ) ) 
  {
    cerr << "Cannot open file " << filename << '\n';
    return NULL;
  } 
  // Read the whole file into the database
  SoSeparator* myGraph = SoDB::readAll(&mySceneInput);
  if( myGraph == NULL ) 
  {
    cerr << "Problem reading file\n";
    return NULL;
  } 
  mySceneInput.closeFile();
  
  myGraph->ref();
  return myGraph;
}


